/******************************************************************************
* Copyright (c) 2026 Advanced Micro Devices, Inc. All Rights Reserved.
* SPDX-License-Identifier: MIT
******************************************************************************/

#ifndef __XPM_DDR_FSM_H__
#define __XPM_DDR_FSM_H__

#include "xstatus.h"
#include "xpm_fsm.h"

#ifdef __cplusplus
extern "C" {
#endif

extern const XPm_Fsm XPmDDRDeviceFsm;

#ifdef __cplusplus
}
#endif

#endif /* __XPM_DDR_FSM_H__ */
